#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: Dop 4Frq Nozaki Chiver
# GNU Radio version: 3.8.1.0

from gnuradio import blocks
from gnuradio import filter
from gnuradio.filter import firdes
from gnuradio import gr
import sys
import signal
from argparse import ArgumentParser
from gnuradio.eng_arg import eng_float, intx
from gnuradio import eng_notation
from gnuradio import uhd
import time

class dop_4frq_nozaki_chiver(gr.top_block):

    def __init__(self):
        gr.top_block.__init__(self, "Dop 4Frq Nozaki Chiver")

        ##################################################
        # Variables
        ##################################################
        self.width = width = 5
        self.tune9M = tune9M = 9595e3
        self.tune8M = tune8M = 8006e3
        self.tune6M = tune6M = 6055e3
        self.tune5M = tune5M = 5006000
        self.tune3M = tune3M = 3925e3
        self.samp_rate = samp_rate = 40000
        self.gain = gain = 60
        self.filter_gain = filter_gain = 30
        self.cutoff_freq = cutoff_freq = 10
        self.center_freq = center_freq = 6e6

        ##################################################
        # Blocks
        ##################################################
        self.uhd_usrp_source_1 = uhd.usrp_source(
            ",".join(("", "")),
            uhd.stream_args(
                cpu_format="fc32",
                args='',
                channels=list(range(0,1)),
            ),
        )
        self.uhd_usrp_source_1.set_time_source('gpsdo', 0)
        self.uhd_usrp_source_1.set_clock_source('gpsdo', 0)
        self.uhd_usrp_source_1.set_center_freq(center_freq, 0)
        self.uhd_usrp_source_1.set_gain(gain, 0)
        self.uhd_usrp_source_1.set_samp_rate(samp_rate*125)
        self.uhd_usrp_source_1.set_time_unknown_pps(uhd.time_spec())
        self.low_pass_filter_0_2 = filter.fir_filter_ccf(
            31250,
            firdes.low_pass(
                filter_gain,
                samp_rate*125,
                cutoff_freq,
                width,
                firdes.WIN_HAMMING,
                6.76))
        self.low_pass_filter_0_1 = filter.fir_filter_ccf(
            31250,
            firdes.low_pass(
                filter_gain,
                samp_rate*125,
                cutoff_freq,
                width,
                firdes.WIN_HAMMING,
                6.76))
        self.low_pass_filter_0_0 = filter.fir_filter_ccf(
            31250,
            firdes.low_pass(
                filter_gain,
                samp_rate*125,
                cutoff_freq,
                width,
                firdes.WIN_HAMMING,
                6.76))
        self.low_pass_filter_0 = filter.fir_filter_ccf(
            31250,
            firdes.low_pass(
                filter_gain,
                samp_rate*125,
                cutoff_freq,
                width,
                firdes.WIN_HAMMING,
                6.76))
        self.freq_xlating_fir_filter_xxx_0_0_1 = filter.freq_xlating_fir_filter_ccc(1, [32], tune5M-center_freq, samp_rate*125)
        self.freq_xlating_fir_filter_xxx_0_0_0 = filter.freq_xlating_fir_filter_ccc(1, [32], tune6M-center_freq, samp_rate*125)
        self.freq_xlating_fir_filter_xxx_0_0 = filter.freq_xlating_fir_filter_ccc(1, [32], tune8M-center_freq, samp_rate*125)
        self.freq_xlating_fir_filter_xxx_0 = filter.freq_xlating_fir_filter_ccc(1, [32], tune3M-center_freq, samp_rate*125)
        self.blocks_file_sink_0_2 = blocks.file_sink(gr.sizeof_gr_complex*1, '/home/hfd/data/data_5.bin', True)
        self.blocks_file_sink_0_2.set_unbuffered(False)
        self.blocks_file_sink_0_1 = blocks.file_sink(gr.sizeof_gr_complex*1, '/home/hfd/data/data_6.bin', True)
        self.blocks_file_sink_0_1.set_unbuffered(False)
        self.blocks_file_sink_0_0 = blocks.file_sink(gr.sizeof_gr_complex*1, '/home/hfd/data/data_8.bin', True)
        self.blocks_file_sink_0_0.set_unbuffered(False)
        self.blocks_file_sink_0 = blocks.file_sink(gr.sizeof_gr_complex*1, '/home/hfd/data/data_3.bin', True)
        self.blocks_file_sink_0.set_unbuffered(False)



        ##################################################
        # Connections
        ##################################################
        self.connect((self.freq_xlating_fir_filter_xxx_0, 0), (self.low_pass_filter_0, 0))
        self.connect((self.freq_xlating_fir_filter_xxx_0_0, 0), (self.low_pass_filter_0_0, 0))
        self.connect((self.freq_xlating_fir_filter_xxx_0_0_0, 0), (self.low_pass_filter_0_1, 0))
        self.connect((self.freq_xlating_fir_filter_xxx_0_0_1, 0), (self.low_pass_filter_0_2, 0))
        self.connect((self.low_pass_filter_0, 0), (self.blocks_file_sink_0, 0))
        self.connect((self.low_pass_filter_0_0, 0), (self.blocks_file_sink_0_0, 0))
        self.connect((self.low_pass_filter_0_1, 0), (self.blocks_file_sink_0_1, 0))
        self.connect((self.low_pass_filter_0_2, 0), (self.blocks_file_sink_0_2, 0))
        self.connect((self.uhd_usrp_source_1, 0), (self.freq_xlating_fir_filter_xxx_0, 0))
        self.connect((self.uhd_usrp_source_1, 0), (self.freq_xlating_fir_filter_xxx_0_0, 0))
        self.connect((self.uhd_usrp_source_1, 0), (self.freq_xlating_fir_filter_xxx_0_0_0, 0))
        self.connect((self.uhd_usrp_source_1, 0), (self.freq_xlating_fir_filter_xxx_0_0_1, 0))

    def get_width(self):
        return self.width

    def set_width(self, width):
        self.width = width
        self.low_pass_filter_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_1.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_2.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))

    def get_tune9M(self):
        return self.tune9M

    def set_tune9M(self, tune9M):
        self.tune9M = tune9M

    def get_tune8M(self):
        return self.tune8M

    def set_tune8M(self, tune8M):
        self.tune8M = tune8M
        self.freq_xlating_fir_filter_xxx_0_0.set_center_freq(self.tune8M-self.center_freq)

    def get_tune6M(self):
        return self.tune6M

    def set_tune6M(self, tune6M):
        self.tune6M = tune6M
        self.freq_xlating_fir_filter_xxx_0_0_0.set_center_freq(self.tune6M-self.center_freq)

    def get_tune5M(self):
        return self.tune5M

    def set_tune5M(self, tune5M):
        self.tune5M = tune5M
        self.freq_xlating_fir_filter_xxx_0_0_1.set_center_freq(self.tune5M-self.center_freq)

    def get_tune3M(self):
        return self.tune3M

    def set_tune3M(self, tune3M):
        self.tune3M = tune3M
        self.freq_xlating_fir_filter_xxx_0.set_center_freq(self.tune3M-self.center_freq)

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.low_pass_filter_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_1.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_2.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.uhd_usrp_source_1.set_samp_rate(self.samp_rate*125)

    def get_gain(self):
        return self.gain

    def set_gain(self, gain):
        self.gain = gain
        self.uhd_usrp_source_1.set_gain(self.gain, 0)

    def get_filter_gain(self):
        return self.filter_gain

    def set_filter_gain(self, filter_gain):
        self.filter_gain = filter_gain
        self.low_pass_filter_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_1.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_2.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))

    def get_cutoff_freq(self):
        return self.cutoff_freq

    def set_cutoff_freq(self, cutoff_freq):
        self.cutoff_freq = cutoff_freq
        self.low_pass_filter_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_1.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_2.set_taps(firdes.low_pass(self.filter_gain, self.samp_rate*125, self.cutoff_freq, self.width, firdes.WIN_HAMMING, 6.76))

    def get_center_freq(self):
        return self.center_freq

    def set_center_freq(self, center_freq):
        self.center_freq = center_freq
        self.freq_xlating_fir_filter_xxx_0.set_center_freq(self.tune3M-self.center_freq)
        self.freq_xlating_fir_filter_xxx_0_0.set_center_freq(self.tune8M-self.center_freq)
        self.freq_xlating_fir_filter_xxx_0_0_0.set_center_freq(self.tune6M-self.center_freq)
        self.freq_xlating_fir_filter_xxx_0_0_1.set_center_freq(self.tune5M-self.center_freq)
        self.uhd_usrp_source_1.set_center_freq(self.center_freq, 0)



def main(top_block_cls=dop_4frq_nozaki_chiver, options=None):
    tb = top_block_cls()

    def sig_handler(sig=None, frame=None):
        tb.stop()
        tb.wait()
        sys.exit(0)

    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)

    tb.start()
    tb.wait()


if __name__ == '__main__':
    main()
